function surfacesFiller(aImarisApplicationID)

% Fill inner surface from clipped surfaces

%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Surfaces Filler" icon="Matlab" tooltip="Fill inner surfaces">
%          <Command>MatlabXT::surfacesFiller(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Copyright (c) 2016, Dominic Filion

% Description : The function allows to fill the inner surfaces when	
% using a Clipping Plane.	
% The user selects the Clipping Plane to produce surfaces and classifies	
% the order of surfaces with the top one in the Surpass Scene to be shown	
% in front of other surfaces (if more than one surface is visible).	
% Only visible (checked) surfaces will be processed using the first visible	
% Clipping Plane.	
% 	


% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Get the Surpass scene
vSurpassScene = vImarisApplication.GetSurpassScene;

% Get Voxel size
aExtendMaxX = vImarisApplication.GetDataSet.GetExtendMaxX;
aExtendMinX = vImarisApplication.GetDataSet.GetExtendMinX;
aSizeX = vImarisApplication.GetDataSet.GetSizeX;
aExtendMaxY = vImarisApplication.GetDataSet.GetExtendMaxY;
aExtendMinY = vImarisApplication.GetDataSet.GetExtendMinY;
aSizeY = vImarisApplication.GetDataSet.GetSizeY;
aExtendMaxZ = vImarisApplication.GetDataSet.GetExtendMaxZ;
aExtendMinZ = vImarisApplication.GetDataSet.GetExtendMinZ;
aSizeZ = vImarisApplication.GetDataSet.GetSizeZ;

vSize = [(aExtendMaxX-aExtendMinX)/aSizeX ...
    (aExtendMaxY-aExtendMinY)/aSizeY ...
    (aExtendMaxZ-aExtendMinZ)/aSizeZ];

sprintf('%f',min(vSize))

% Get the first Clipping plane Selected
vClippingPlane = [];
for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if vImarisApplication.GetFactory.IsClippingPlane(vDataItem)
        vClippingPlane = vImarisApplication.GetFactory.ToClippingPlane(vDataItem);
        if vClippingPlane.GetVisible
            break
        else
            vClippingPlane = [];
        end
    end
end
if isempty(vClippingPlane)
    errordlg('Please select a Clipping Plane!');
    return  % stops processing if no Clipping Plane selected
end

% Get values from the Clipping Plane
vClippingPlaneValues=vClippingPlane.GetOrientationAxisAngle;
vAxis = vClippingPlaneValues.mAxisXYZ;
vAngle = vClippingPlaneValues.mAngle;
vPosition = vClippingPlane.GetPosition;

% Variable initialization
vSurface = [];
surfaceVisibleIdx = 0;

for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vSurface = vImarisApplication.GetFactory.ToSurfaces(vDataItem);
        if vSurface.GetVisible  % Process only if the surface is selected
            % Run the algorithm for all different surfaces
            vIds = vSurface.GetIds();
            vSurfaceGroup = vImarisApplication.GetFactory.CreateDataContainer;
            name = char(vSurface.GetName());
            
            sprintf('Intersection from %s',name)
            vSurfaceGroup.SetName(sprintf('Intersection from %s',name));
            for aSurfaceIndex = 0:size(vIds,1)-1
                
                % Process the surface if it is the current Time point
                if vSurface.GetTimeIndex(aSurfaceIndex) == vImarisApplication.GetVisibleIndexT
                    
                    % Get the surfaces components
                    aVertices = vSurface.GetVertices(aSurfaceIndex);
                    aFaces = vSurface.GetTriangles(aSurfaceIndex);
                    aNormals = vSurface.GetNormals(aSurfaceIndex);
                    
                    % Find the clipping plane position and angle
                    v = [0;0;1];  % Initial position of the Clipping Plane
                    vr = v.*cos(vAngle) + cross(vAxis,v).*sin(vAngle)...
                        + vAxis.*(vAxis'*v).*(1-cos(vAngle)); % Orthogonal vector vr
                    
                    % find the volume behind the Clipping plane
                    ind = aVertices*vr - vPosition'*vr<0;  % index of vertices behind the plane
                    nV = aVertices;  % Copy vertices
                    nV(~ind) = nan; % Set all vertices "not ind" to Not a Number (NaN)
                    rFaces = aFaces(:);  % Use the faces as one column
                    
                    % Values of vertices from the faces (as one column)
                    v = nV(rFaces+1,:);
                    vInd = any(isnan(v),2);  % index of
                    rInd = reshape(vInd,size(aFaces));
                    IndAny = any(rInd,2);
                    IndAll = all(rInd,2);
                    
                    % Faces at intersection of the clipping plane
                    nFaces = aFaces(IndAny&~IndAll,:);
                    
                    if not(isempty(nFaces))
                        disp('Find the intersection');
                        % find the center point of Faces and the new normals
                        newP = zeros(size(nFaces,1),3);
                        newN = zeros(size(nFaces,1),3);
                        for nFaceID = 1:size(nFaces,1)
                            newP(nFaceID,:) = mean(aVertices(nFaces(nFaceID,:)+1,:),1);
                            newN(nFaceID,:) = mean(aNormals(nFaces(nFaceID,:)+1,:),1);
                            newN(nFaceID,:) = cross(vr',cross(newN(nFaceID,:),vr'));
                        end
                        lm = vPosition'*vr - vr'*newP';
                        newP = newP + lm'*vr';
                        
                        % translate and rotate the surfaces into the
                        % XY plane to simplify calculations
                        
                        newP = newP - repmat(vPosition',[size(newP,1) 1]);
                        nvAxis = repmat(vAxis,1,size(newP,1))';
                        newP = newP.*cos(-vAngle) + ...
                            cross(nvAxis,newP,2).*sin(-vAngle) + ...
                            nvAxis.*repmat(dot(nvAxis',newP'),[3 1])'.*(1-cos(-vAngle));
                        
                        % rotate the normals
                        newN = newN.*cos(-vAngle) + ...
                            cross(nvAxis,newN,2).*sin(-vAngle) + ...
                            nvAxis.*repmat(dot(nvAxis',newN'),[3 1])'.*(1-cos(-vAngle));
                        
                        % find the surface groups
                        [groupIND,dist] = clusterXY(newP,-1);
                        sprintf('%f',dist)
                        
                        % NOTE : You might want to change the value of dist
                        % for a smaller value.  This value is used later to
                        % product a grid and check inside surface values
                        
                        % Check surface range and if surface is Inside
                        rangeM = zeros(max(groupIND),4);
                        rangeMinside = zeros(max(groupIND),4);
                        insideP = newP - dist*newN; % produce a smaller surface if inside
                        for groupID = 1:max(groupIND)
                            gInd = groupIND == groupID;
                            rangeM(groupID,:) = [min(newP(gInd,1:2)) max(newP(gInd,1:2))];
                            rangeMinside(groupID,:) = [min(insideP(gInd,1:2)) max(insideP(gInd,1:2))];
                        end
                        
                        % Identify inside surfaces
                        inside = rangeMinside < rangeM;
                        insideID = find(all(inside(:,1:2),2));
                        
                        inRangeM = zeros(max(groupIND));
                        for groupID = 1:max(groupIND)
                            inRangeM(groupID,:) = rangeM(groupID,1) > rangeM(:,1) &...
                                rangeM(groupID,2) > rangeM(:,2) &...
                                rangeM(groupID,3) < rangeM(:,3) &...
                                rangeM(groupID,4) < rangeM(:,4);
                        end
                        
                        % Change the groupIND value
                        if not(isempty(insideID))
                            for ID = 1:size(insideID,1)
                                gInd = groupIND == insideID(ID);
                                if not(isempty(find(inRangeM(insideID(ID),:), 1)))
                                    gValue = find(inRangeM(insideID(ID),:));
                                    groupIND(gInd) = gValue(1);
                                end
                            end
                        end
                        
                        % Process all group
                        groups = unique(groupIND);
                        for gr = 1:size(groups,1)
                            groupIndex = groupIND == groups(gr);
                            points = newP(groupIndex,1:2);
                            normals = newN(groupIndex,1:2);
                            
                            % Start the grid
                            minXY = min(points);
                            maxXY = max(points);
                            [X,Y] = meshgrid(minXY(1):dist:maxXY(1),minXY(2):dist:maxXY(2));
                            X = reshape(X,size(X,1)*size(X,2),1);
                            Y = reshape(Y,size(Y,1)*size(Y,2),1);
                            
                            PX = [points(:,1);X];
                            PY = [points(:,2);Y];
                            PX = double(PX);
                            PY = double(PY);
                            XY = [X Y];
                            DT = delaunayTriangulation(PX,PY);
                            
                            % find the inside points
                            indsel = false(size(X));
                            for j = 1:size(XY,1)
                                [~,indm] = min(sum((points - repmat(XY(j,:),[size(points,1) 1])).^2,2));
                                indsel(j) = dot((points(indm,:) - XY(j,:)),normals(indm,:)) > 0;
                            end
                            indselnew = [true(size(points,1),1);indsel];
                            
                            % remove triangles
                            nV = DT.Points;
                            nV(~indselnew) = nan;
                            vTriangles = DT.ConnectivityList;
                            rTriangles = vTriangles(:);
                            v = nV(rTriangles,:);
                            vInd = any(isnan(v),2);
                            rInd = reshape(vInd,size(vTriangles));
                            IndAny = any(rInd,2);
                            
                            nTriangles = vTriangles(~IndAny,:);
                            vVertices = [DT.Points zeros(size(DT.Points,1),1)];
                            
                            % translate and rotate the vertices
                            nvAxis = repmat(vAxis,1,size(vVertices,1))';
                            vVertices = vVertices.*cos(vAngle) + ...
                                cross(nvAxis,vVertices,2).*sin(vAngle) + ...
                                nvAxis.*repmat(dot(nvAxis',vVertices'),[3 1])'.*(1-cos(vAngle));
                            vVertices = vVertices + repmat(vPosition',[size(vVertices,1) 1]);
                            
                            vNormals = repmat(vr',[size(vVertices,1) 1]);
                            
                            % Move back the Surface position by tenth of 
                            % the smaller voxel distance to show
                            % surfaces in the position from the surpass
                            % scene and help visualization
                            vVertices = vVertices - 0.1*min(vSize)*surfaceVisibleIdx*vNormals;
                            
                            % Create the new surface
                            newSurfaces = vImarisApplication.GetFactory.CreateSurfaces;
                            newSurfaces.AddSurface(vVertices, nTriangles-1, vNormals, 0);
                            vRGBA = vSurface.GetColorRGBA;
                            [R,G,B] = rgba2rgb(vRGBA);
                            newSurfaces.SetColorRGBA(rgb2rgba(R,G,B));
                            newSurfaces.SetName(sprintf('Intersection with ID %d',vIds(aSurfaceIndex + 1)));
                            vSurfaceGroup.AddChild(newSurfaces, -1);
                        end
                    end
                end
            end
            vSurpassScene.AddChild(vSurfaceGroup, -1);
            surfaceVisibleIdx = surfaceVisibleIdx + 1;
        end
    end
end
% move the clipping plane to a position which will not hide the
% new surfaces
vPosition = vPosition + 0.1*min(vSize)*vr;
vClippingPlane.SetPosition(vPosition);

% Return the error that No Surfaces are selected
if isempty(vSurface)
    errordlg('Please Create a surface or select one!');
    return
end

%--------------------------------------------------------------------------
    function [R,G,B,A] = rgba2rgb(rgba)
        
        if rgba < 0
            rgba = rgba + 2^32;
        end
        str = dec2bin(rgba);
        strz = sprintf('%032d',0);
        strz(end-length(str)+1:end) = str;
        A = bin2dec(strz(1:8));
        B = bin2dec(strz(9:16));
        G = bin2dec(strz(17:24));
        R = bin2dec(strz(25:32));
    end
%--------------------------------------------------------------------------
    function rgba = rgb2rgba(R,G,B)
        
        rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
            str2double(dec2bin(G)),str2double(dec2bin(R))));
    end
%--------------------------------------------------------------------------
    function [groupIND,dist] = clusterXY(xy,dist)
        
        D = newpdist(xy);
        if ~isempty(D)
            if dist < 0
                di = logical(diag(ones(size(D,1),1)));
                D(di) = nan;
                dist = 5*mean(min(D));
                D(di) = 0;
            end
            
            groupIND = zeros(size(D,1),1);
            list_ingroup = false(size(xy,1),1);
            list_done = false(size(xy,1),1);
            groupnum = 1;
            while ~all(list_done)
                todo = find(~list_done);
                todo = todo(1);
                currentgroupnum = groupnum;
                cindex = [];
                while groupnum == currentgroupnum
                    indc = find(D(todo,:) < dist);
                    cindex = checkIndex(cindex,indc);
                    list_ingroup(indc) = 1;
                    list_done(todo) = 1;
                    islast = find(cindex == todo);
                    if islast == length(cindex)
                        groupnum = groupnum + 1;
                    else
                        todo = cindex(islast + 1);
                    end
                end
                for ii = 1:size(cindex,2)
                    groupIND(cindex(ii)) = currentgroupnum;
                end
            end
            
        else
            groupIND = [];
        end
        
        %--------------------------------------------------------------------------
        % check if index is part of the list and update list
        function newind = checkIndex(oldind,ind)
            
            isin = false(length(ind),1);
            for i = 1:length(ind)
                isin(i) = any(oldind == ind(i));
            end
            newind = [oldind ind(~isin)];
        end
        %--------------------------------------------------------------------------
        function D = newpdist(xy)
            
            D = zeros(size(xy,1));
            for i = 1:size(xy,1)
                p1 = repmat(xy(i,:),[size(xy,1) 1]);
                D(i,:) = sqrt(sum((xy - p1).^2,2));
            end
        end
        
    end



end
